import {
  dispatchEventFromFixture,
  getElementFromFixture,
  mock,
  Mock,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { AuthenticationService } from 'authentication';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { UserProfileButtonContainerComponent } from './user-profile.button-container.component';

describe('UserProfileButtonContainerComponent', () => {
  let component: UserProfileButtonContainerComponent;
  let fixture: ComponentFixture<UserProfileButtonContainerComponent>;

  const authenticationService: Mock<AuthenticationService> = mock(AuthenticationService);

  const dropDownButton: string = getDataTestIdOf('drop-down-button');
  const logout: string = getDataTestIdOf('logout');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [UserProfileButtonContainerComponent],
      imports: [RouterTestingModule],
      providers: [
        {
          provide: AuthenticationService,
          useValue: authenticationService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(UserProfileButtonContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call authService to get current user initials', () => {
      component.ngOnInit();

      expect(authenticationService.getCurrentUserInitials).toHaveBeenCalled();
    });
  });

  describe('button', () => {
    it('should call showDropDown on click', () => {
      component.showDropDown = jest.fn();

      dispatchEventFromFixture(fixture, dropDownButton, 'click');

      expect(component.showDropDown).toHaveBeenCalled();
    });

    it('should show initials', () => {
      const userInitials: string = 'AV';
      component.currentUserInitials = userInitials;

      fixture.detectChanges();

      const buttonElement: HTMLElement = getElementFromFixture(fixture, dropDownButton);
      expect(buttonElement.textContent.trim()).toEqual('AV');
    });
  });

  describe('abmelden', () => {
    it('should call authService logout', () => {
      dispatchEventFromFixture(fixture, logout, 'click');

      expect(authenticationService.logout).toHaveBeenCalled();
    });
  });
});
