import { CdkStepperModule } from '@angular/cdk/stepper';
import { CommonModule } from '@angular/common';
import { Component, HostBinding, effect, signal } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import {
  AttachmentComponent,
  AttachmentIconComponent,
  AttachmentWrapperComponent,
  BescheidGenerateIconComponent,
  BescheidUploadIconComponent,
  ButtonCardComponent,
  ButtonComponent,
  CloseIconComponent,
  ErrorMessageComponent,
  FileIconComponent,
  FileUploadButtonComponent,
  InstantSearchComponent,
  OfficeIconComponent,
  RadioButtonCardComponent,
  SaveIconComponent,
  SendIconComponent,
  SpinnerIconComponent,
  StampIconComponent,
  TextInputComponent,
  TextareaComponent,
} from '@ods/system';

import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { Resource } from '@ngxp/rest';
import {
  InstantSearchQuery,
  InstantSearchResult,
} from 'libs/design-system/src/lib/instant-search/instant-search/instant-search.model';
import { BescheidDialogExampleComponent } from './components/bescheid-dialog/bescheid-dialog.component';
import { BescheidPaperComponent } from './components/bescheid-paper/bescheid-paper.component';
import { BescheidStepperComponent } from './components/bescheid-stepper/bescheid-stepper.component';
import { CustomStepperComponent } from './components/cdk-demo/custom-stepper.component';

@Component({
  standalone: true,
  imports: [
    CommonModule,
    AttachmentComponent,
    AttachmentWrapperComponent,
    ButtonComponent,
    ButtonCardComponent,
    FileUploadButtonComponent,
    RouterModule,
    CdkStepperModule,
    CustomStepperComponent,
    BescheidDialogExampleComponent,
    BescheidStepperComponent,
    BescheidPaperComponent,
    RadioButtonCardComponent,
    ReactiveFormsModule,
    InstantSearchComponent,
    OfficeIconComponent,
    SaveIconComponent,
    SendIconComponent,
    StampIconComponent,
    CloseIconComponent,
    AttachmentIconComponent,
    BescheidGenerateIconComponent,
    BescheidUploadIconComponent,
    SpinnerIconComponent,
    FileIconComponent,
    TextareaComponent,
    TextInputComponent,
    TextareaComponent,
    ErrorMessageComponent,
  ],
  selector: 'app-root',
  templateUrl: './app.component.html',
})
export class AppComponent {
  darkMode = signal<boolean>(JSON.parse(window.localStorage.getItem('darkMode') ?? 'false'));

  @HostBinding('class.dark') get mode() {
    return this.darkMode();
  }

  title = 'demo';

  instantSearchItems: InstantSearchResult<Resource>[] = [
    {
      title: 'Landeshauptstadt Kiel - Ordnungsamt, Gewerbe- und Schornsteinfegeraufsicht',
      description: 'Fabrikstraße  8-10, 24103 Kiel',
    },
    {
      title: 'Amt für Digitalisierung, Breitband und Vermessung Nürnberg Außenstelle Hersbruck',
      description: 'Rathausmarkt 7, Hersbruck',
    },
    {
      title: 'Amt für Digitalisierung, Breitband und Vermessung Stuttgart',
      description: 'Rathausmarkt 7, Stuttgart',
    },
    {
      title: 'Amt für Digitalisierung, Breitband und Vermessung Ulm',
      description: 'Rathausmarkt 7, Ulm',
    },
  ];
  instantSearchFormControl = new FormControl(EMPTY_STRING);

  getInstantSearchResults() {
    if (this.instantSearchFormControl.value.length < 2) return [];
    return this.instantSearchItems.filter((item) =>
      item.title.toLowerCase().includes(this.instantSearchFormControl.value.toLowerCase()),
    );
  }

  selectSearchResult(result: InstantSearchResult<Resource>) {
    console.log(result);
  }

  exampleForm = new FormGroup({
    exampleName: new FormControl('bewilligt'),
  });

  saveForm = new FormGroup({
    saveForm: new FormControl(),
  });

  onDownloadClick = () => window.alert('download clicked');

  constructor() {
    effect(() => {
      window.localStorage.setItem('darkMode', JSON.stringify(this.darkMode()));
    });
  }

  public onSearchQueryChanged(searchQuery: InstantSearchQuery) {
    console.info('Search query: %o', searchQuery);
  }
}
