import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getElementFromFixtureByType,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { singleCold } from '../../../../../tech-shared/test/marbles';
import {
  createOrganisationseinheit,
  createOrganisationseinheitState,
} from '../../../../test/user/user';
import { SecondaryButtonComponent } from '../../shared/secondary-button/secondary-button.component';
import { SpinnerComponent } from '../../shared/spinner/spinner.component';
import { Organisationseinheit } from '../../user/user.model';
import { UserService } from '../../user/user.service';
import { OrganisationseinheitContainerComponent } from './organisationseinheit-container.component';
import { OrganisationseinheitFormComponent } from './organisationseinheit-form/organisationseinheit-form.component';
import { OrganisationseinheitListComponent } from './organisationseinheit-list/organisationseinheit-list.component';

describe('OrganisationseinheitContainerComponent', () => {
  let component: OrganisationseinheitContainerComponent;
  let fixture: ComponentFixture<OrganisationseinheitContainerComponent>;

  const userService: Mock<UserService> = mock(UserService);

  const dialogOpenButtonSelector: string = getDataTestIdOf(
    'organisationseinheit-open-dialog-button',
  );
  const spinnerSelector: string = getDataTestIdOf('organisationseinheit-spinner');

  const organisationseinheitItems: Organisationseinheit[] = [createOrganisationseinheit()];

  let formComponent: OrganisationseinheitFormComponent;
  let listComponent: OrganisationseinheitListComponent;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        OrganisationseinheitContainerComponent,
        MockComponent(SecondaryButtonComponent),
        MockComponent(OrganisationseinheitFormComponent),
        MockComponent(OrganisationseinheitListComponent),
        MockComponent(SpinnerComponent),
      ],
      providers: [{ provide: UserService, useValue: userService }],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationseinheitContainerComponent);
    component = fixture.componentInstance;

    userService.getOrganisationseinheitState = jest
      .fn()
      .mockReturnValue(of(createOrganisationseinheitState(organisationseinheitItems)));
    fixture.detectChanges();

    formComponent = getElementFromFixtureByType(fixture, OrganisationseinheitFormComponent);
    listComponent = getElementFromFixtureByType(fixture, OrganisationseinheitListComponent);
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should open form on new organisationseinheit button', () => {
    formComponent.open = jest.fn();

    dispatchEventFromFixture(fixture, dialogOpenButtonSelector, 'clickEmitter');

    expect(formComponent.open).toHaveBeenCalled();
  });

  describe('organisationseinheit list', () => {
    it('should open form for editing on editOrganisationseinheit event', () => {
      const organisationseinheit: Organisationseinheit = organisationseinheitItems[0];
      formComponent.openEdit = jest.fn();

      listComponent.editOrganisationseinheit.emit(organisationseinheit);

      expect(formComponent.openEdit).toHaveBeenCalledWith(organisationseinheit);
    });

    it('should call deleteOrganisationseinheit form on deleteOrganisationseinheit event', () => {
      const organisationseinheit: Organisationseinheit = organisationseinheitItems[0];
      component.delete = jest.fn();

      listComponent.deleteOrganisationseinheit.emit(organisationseinheit);

      expect(component.delete).toHaveBeenCalledWith(organisationseinheit);
    });
  });

  describe('delete', () => {
    const organisationseinheit: Organisationseinheit = organisationseinheitItems[0];

    beforeEach(() => {
      userService.deleteOrganisationseinheit = jest.fn().mockReturnValue(singleCold(true));
    });

    it('should call service method', () => {
      component.delete(organisationseinheit);

      expect(userService.deleteOrganisationseinheit).toHaveBeenCalledWith(organisationseinheit.id);
    });

    it('should assign delete progress observable', () => {
      component.delete(organisationseinheit);

      expect(component.deleteInProgress$).toBeObservable(singleCold(true));
    });
  });

  describe('spinner', () => {
    it('should not show if delete in not progress', () => {
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, spinnerSelector);
    });
    it('should show if delete in progress', () => {
      component.deleteInProgress$ = of(true);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, spinnerSelector);
    });
  });
});
