import { dispatchEventFromFixture, getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MoreItemButtonComponent } from './more-item-button.component';

describe('MoreItemButtonComponent', () => {
  let component: MoreItemButtonComponent;
  let fixture: ComponentFixture<MoreItemButtonComponent>;

  const buttonSelector: string = 'button';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [MoreItemButtonComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(MoreItemButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should show label', () => {
    const text: string = 'test-text';
    component.label = text;

    fixture.detectChanges();

    const buttonElement: HTMLButtonElement = getElementFromFixture(fixture, buttonSelector);
    expect(buttonElement.textContent.trim()).toEqual(text);
  });

  it.each([false, true])('should use disabled "%s"', (disabled) => {
    component.disabled = disabled;

    fixture.detectChanges();

    const buttonElement: HTMLButtonElement = getElementFromFixture(fixture, buttonSelector);
    expect(buttonElement.disabled).toBe(disabled);
  });

  it('should emit clickEmitter', () => {
    component.clickEmitter.emit = jest.fn();

    dispatchEventFromFixture(fixture, buttonSelector, 'click');

    expect(component.clickEmitter.emit).toHaveBeenCalled();
  });
});
