import { faker } from '@faker-js/faker';
import { NetworkError } from '@keycloak/keycloak-admin-client';
import type GroupRepresentation from '@keycloak/keycloak-admin-client/lib/defs/groupRepresentation';
import {
  Organisationseinheit,
  OrganisationseinheitError,
  OrganisationseinheitErrorType,
  OrganisationseinheitState,
} from '../../src/lib/user/user.model';

export function createOrganisationseinheit(): Organisationseinheit {
  return {
    id: faker.random.alphaNumeric(16),
    name: faker.name.jobTitle(),
    organisationseinheitIds: [faker.random.numeric(10), faker.random.numeric(10)],
  };
}

export function createGroupRepresentation(
  organisationseinheit: Organisationseinheit = createOrganisationseinheit(),
): GroupRepresentation {
  return {
    id: organisationseinheit.id,
    name: organisationseinheit.name,
    attributes: {
      organisationseinheitId: organisationseinheit.organisationseinheitIds,
    },
  };
}

export function createOrganisationseinheitState(
  organisationseinheitItems: Organisationseinheit[] = [createOrganisationseinheit()],
): OrganisationseinheitState {
  return {
    organisationseinheitItems: organisationseinheitItems,
    loading: false,
    updateRequired: false,
  };
}

export function createOrganisationseinheitError(
  errorType: OrganisationseinheitErrorType = OrganisationseinheitErrorType.NAME_MISSING,
): OrganisationseinheitError {
  return {
    errorType,
    detail: faker.lorem.sentence(10),
  };
}

export function createNetworkError(status: number, errorMessage: string | undefined): NetworkError {
  return new NetworkError('...', {
    response: { status } as Response,
    responseData: {
      errorMessage,
    },
  });
}
