import { Mock, mock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { MatDialogModule } from '@angular/material/dialog';
import { HttpUnauthorizedInterceptor } from './http-unauthorized.interceptor';
import { HttpErrorResponse, HttpHandler, HttpRequest } from '@angular/common/http';
import { Subject, isEmpty } from 'rxjs';
import { AuthenticationService } from './authentication.service';

describe('HttpUnauthorizedInterceptor', () => {
  let interceptor: HttpUnauthorizedInterceptor;

  const authenticationService: Mock<AuthenticationService> = mock(AuthenticationService);

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [MatDialogModule],
      providers: [
        HttpUnauthorizedInterceptor,
        {
          provide: AuthenticationService,
          useValue: authenticationService,
        },
      ],
    }),
  );

  beforeEach(() => {
    interceptor = TestBed.inject(HttpUnauthorizedInterceptor);
  });

  it('should be created', () => {
    expect(interceptor).toBeTruthy();
  });

  describe('intercept', () => {
    const error: HttpErrorResponse = new HttpErrorResponse({});

    const handleSubject: Subject<any> = new Subject();
    const httpHandler: HttpHandler = { handle: () => handleSubject };
    const request: HttpRequest<unknown> = new HttpRequest('GET', '/test');

    it('should call handleError with error', () => {
      interceptor.handleError = jest.fn();

      interceptor.intercept(request, httpHandler).subscribe();
      handleSubject.error(error);

      expect(interceptor.handleError).toHaveBeenCalledWith(error);
    });
  });

  describe('handleError', () => {
    describe('on unauthorized status', () => {
      const unauthorizedError: any = new HttpErrorResponse({ status: 401 });

      it('should call logout on authService ', () => {
        interceptor.handleError(unauthorizedError);

        expect(authenticationService.logout).toHaveBeenCalled();
      });

      it('should return EMPTY', (done) => {
        interceptor
          .handleError(unauthorizedError)
          .pipe(isEmpty())
          .subscribe((res) => {
            expect(res).toBeTruthy;
            done();
          });
      });
    });

    it('should rethrow error if not unauthorized status', () => {
      const anyError: any = new HttpErrorResponse({ status: 500 });

      expect(() => interceptor.handleError(anyError)).toThrowError();
    });
  });
});
