import { CommandOrder, CommandResource, CommandService } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { CollaborationListResourceService } from './collaboration-list-resource.service';
import { CollaborationListLinkRel } from './collaboration.linkrel';
import { Collaboration, CollaborationListResource } from './collaboration.model';

@Injectable()
export class CollaborationService {
  showRequestForm$: BehaviorSubject<boolean> = new BehaviorSubject(false);

  constructor(
    private listService: CollaborationListResourceService,
    private commandService: CommandService,
  ) {}

  public getList(): Observable<StateResource<CollaborationListResource>> {
    return this.listService.getList();
  }

  public isRequestFormVisible(): Observable<boolean> {
    return this.showRequestForm$.asObservable();
  }

  public showRequestForm(): void {
    this.showRequestForm$.next(true);
  }

  public hideRequestForm(): void {
    this.showRequestForm$.next(false);
  }

  public create(
    listResource: CollaborationListResource,
    collaborationRequest: Collaboration,
  ): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps({
      linkRel: CollaborationListLinkRel.CREATE_COLLABORATION_REQUEST,
      resource: listResource,
      command: {
        order: CommandOrder.CREATE_COLLABORATION_REQUEST,
        body: collaborationRequest,
      },
      snackBarMessage: 'Die Zuarbeit wurde angefragt.',
    });
  }
}
