import { OrganisationsEinheitResource } from '@alfa-client/collaboration-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { CollaborationListResource } from 'libs/collaboration-shared/src/lib/collaboration.model';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { OrganisationsEinheitResourceSearchService } from 'libs/collaboration-shared/src/lib/organisations-einheit-resource-search.service';
import { Observable } from 'rxjs';

@Component({
  selector: 'alfa-collaboration-in-vorgang-container',
  templateUrl: './collaboration-in-vorgang-container.component.html',
})
export class CollaborationInVorgangContainerComponent implements OnInit, OnDestroy {
  public collaborationStateListResource$: Observable<StateResource<CollaborationListResource>>;
  public isRequestFormVisible$: Observable<boolean>;
  public selectedOrganisationsEinheit$: Observable<OrganisationsEinheitResource>;

  constructor(
    private readonly service: CollaborationService,
    private readonly searchService: OrganisationsEinheitResourceSearchService,
  ) {}

  ngOnInit(): void {
    this.collaborationStateListResource$ = this.service.getList();
    this.isRequestFormVisible$ = this.service.isRequestFormVisible();
    this.selectedOrganisationsEinheit$ = this.searchService.getSelectedResult();
  }

  public showRequestForm(): void {
    this.service.showRequestForm();
  }

  public hideRequestForm(): void {
    this.service.hideRequestForm();
  }

  ngOnDestroy(): void {
    this.searchService.clearSelectedResult();
  }
}
