import { Collaboration, CollaborationListResource } from '@alfa-client/collaboration-shared';
import { CommandResource, tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { HttpError, StateResource, createEmptyStateResource } from '@alfa-client/tech-shared';
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { Observable, of } from 'rxjs';
import { CollaborationRequestFormService } from './collaboration.request.formservice';

@Component({
  selector: 'alfa-collaboration-request-form',
  templateUrl: './collaboration-request-form.component.html',
  providers: [CollaborationRequestFormService],
})
export class CollaborationRequestFormComponent {
  @Output() public hide: EventEmitter<void> = new EventEmitter<void>();
  @Output() public showResult: EventEmitter<Collaboration> = new EventEmitter<Collaboration>();

  @Input() public set collaborationListResource(
    collaborationListResource: CollaborationListResource,
  ) {
    this.formService.setListResource(collaborationListResource);
  }

  public submitInProgress$: Observable<StateResource<CommandResource | HttpError>> = of(
    createEmptyStateResource<CommandResource>(),
  );

  constructor(readonly formService: CollaborationRequestFormService) {}

  public readonly formServiceClass = CollaborationRequestFormService;

  public submit(): void {
    this.submitInProgress$ = this.doSubmit();
  }

  doSubmit(): Observable<StateResource<CommandResource>> {
    return this.formService
      .submit()
      .pipe(
        tapOnCommandSuccessfullyDone(() => this.showResult.emit(this.formService.getFormValue())),
      );
  }
}
