/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TechSharedModule, createEmptyStateResource } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  AbstractControl,
  FormGroupDirective,
  ReactiveFormsModule,
  UntypedFormBuilder,
} from '@angular/forms';
import { FileUploadButtonComponent, SpinnerIconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { FileUploadEditorComponent } from './file-upload-editor.component';

describe('FileUploadEditorComponent', () => {
  let component: FileUploadEditorComponent;
  let fixture: ComponentFixture<FileUploadEditorComponent>;

  const buttonTestId: string = getDataTestIdOf('Ein_Label-file-upload-button');
  const fb = new UntypedFormBuilder();
  const formGroupDirective = new FormGroupDirective([], []);
  formGroupDirective.form = fb.group({
    attachments: fb.control(null),
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        FileUploadEditorComponent,
        MockComponent(SpinnerIconComponent),
        MockComponent(FileUploadButtonComponent),
      ],
      imports: [ReactiveFormsModule, TechSharedModule],
      providers: [
        {
          provide: FormGroupDirective,
          useValue: formGroupDirective,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(FileUploadEditorComponent);
    component = fixture.componentInstance;
    component.parentFormArrayName = 'attachments';
    component.uploadInProgress = createEmptyStateResource();
    component.label = 'Ein Label';
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should have upload button', () => {
    const element: HTMLElement = getElementFromFixture(fixture, buttonTestId);

    expect(element).toBeInstanceOf(HTMLElement);
  });

  describe('after OnInit', () => {
    it('should get fileLinkControls', () => {
      expect(component.fileLinkControls).toBeInstanceOf(AbstractControl);
    });
  });
});
