import { getMessageForInvalidParam, InvalidParam } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ValidationMessageCode } from 'libs/tech-shared/src/lib/validation/tech.validation.messages';
import { createInvalidParam } from 'libs/tech-shared/test/error';
import { ValidationErrorComponent } from './validation-error.component';

describe('ValidationErrorComponent', () => {
  let component: ValidationErrorComponent;
  let fixture: ComponentFixture<ValidationErrorComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ValidationErrorComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(ValidationErrorComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('get message from invalidParam', () => {
    const fieldLabel: string = 'Field Label';
    const invalidParam: InvalidParam = {
      ...createInvalidParam(),
      reason: ValidationMessageCode.FIELD_SIZE,
    };

    it('should contain', () => {
      const msg: string = getMessageForInvalidParam(fieldLabel, invalidParam);

      expect(msg).toContain('muss mindestens');
    });

    it('should set field label', () => {
      const msg: string = getMessageForInvalidParam(fieldLabel, invalidParam);

      expect(msg).toContain(fieldLabel);
    });

    it('should replace min param', () => {
      const msg: string = getMessageForInvalidParam(fieldLabel, {
        ...invalidParam,
        constraintParameters: [{ name: 'min', value: '3' }],
      });

      expect(msg).toContain('3');
    });
  });
});
