import { CdkTrapFocus } from '@angular/cdk/a11y';
import { CommonModule } from '@angular/common';
import { Component, ElementRef, HostListener, Input, ViewChild } from '@angular/core';
import { twMerge } from 'tailwind-merge';

@Component({
  selector: 'ods-popup',
  standalone: true,
  imports: [CommonModule, CdkTrapFocus],
  template: `<div class="relative w-fit">
    <button
      class="w-fit outline-2 outline-offset-2 outline-focus"
      [ngClass]="[twMerge('w-fit outline-2 outline-offset-2 outline-focus', buttonClass)]"
      (click)="togglePopup()"
      [attr.aria-expanded]="isPopupOpen"
      aria-haspopup="true"
      [attr.aria-label]="label"
      data-test-id="popup-button"
      #button
    >
      <ng-content select="[button]" />
    </button>
    <ul
      *ngIf="isPopupOpen"
      class="max-h-120 animate-fadeIn absolute min-w-44 max-w-80 overflow-y-auto rounded shadow-lg shadow-grayborder focus:outline-none"
      [ngClass]="alignTo === 'left' ? 'right-0' : 'left-0'"
      role="dialog"
      aria-modal="true"
      cdkTrapFocus
      [cdkTrapFocusAutoCapture]="true"
    >
      <ng-content />
    </ul>
  </div>`,
})
export class PopupComponent {
  @Input() alignTo: 'left' | 'right' = 'left';
  @Input() label: string = '';
  @Input() buttonClass: string = '';

  isPopupOpen: boolean = false;
  twMerge = twMerge;

  @ViewChild('button') buttonRef: ElementRef<HTMLButtonElement>;

  @HostListener('document:keydown', ['$event'])
  onKeydownHandler(e: KeyboardEvent): void {
    if (this.isPopupClosed()) return;
    if (this.isEscapeKey(e)) this.closePopupAndFocusButton();
  }

  @HostListener('document:click', ['$event'])
  onClickHandler(e: MouseEvent): void {
    if (this.isPopupClosed()) return;
    if (!this.buttonRef.nativeElement.contains(e.target as HTMLElement)) {
      this.closePopupAndFocusButton();
    }
  }

  togglePopup(): void {
    this.isPopupOpen = !this.isPopupOpen;
  }

  closePopupAndFocusButton(): void {
    this.isPopupOpen = false;
    this.buttonRef.nativeElement.focus();
  }

  isEscapeKey(e: KeyboardEvent): boolean {
    return e.key === 'Escape';
  }

  isPopupClosed(): boolean {
    return !this.isPopupOpen;
  }
}
