import { Router, UrlTree } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { faker } from '@faker-js/faker';
import { OzgCloudUrlSerializer } from './ozgcloud-url-serializer';

import { TestBed } from '@angular/core/testing';
import * as NavigationUtil from './navigation.util';

describe('OzgCloudUrlSerializer', () => {
  let serializer: OzgCloudUrlSerializer;
  let router: Router;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [RouterTestingModule],
    });

    router = TestBed.inject(Router);
    serializer = new OzgCloudUrlSerializer();
  });

  it('should be created', () => {
    expect(serializer).toBeTruthy();
  });

  describe('parse', () => {
    it('should return URLTree', () => {
      const url: string = faker.internet.url();

      const urlTree: UrlTree = serializer.parse(url);

      expect(urlTree).toBeInstanceOf(UrlTree);
    });
  });

  describe('serialize', () => {
    it('should call revertDoubleEncodingOfPercentageSign', () => {
      const spy: jest.SpyInstance<string, [text: string]> = jest.spyOn(
        NavigationUtil,
        'revertDoubleEncodingOfPercentageSign',
      );
      const urlTree: UrlTree = router.createUrlTree(['/']);

      serializer.serialize(urlTree);

      expect(spy).toHaveBeenCalled();
    });

    it('should ensure percentage sign is not double encoded', () => {
      const urlTree: UrlTree = router.createUrlTree(['/', '%25']);

      const path: string = serializer.serialize(urlTree);

      expect(path).toBe('/%25');
    });
  });
});
