/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NgControl, ReactiveFormsModule, UntypedFormControl } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { MockComponent } from 'ng-mocks';
import { ValidationErrorComponent } from '../../validation-error/validation-error.component';
import { TextEditorComponent } from './text-editor.component';

describe('TextEditorComponent', () => {
  let component: TextEditorComponent;
  let fixture: ComponentFixture<TextEditorComponent>;

  const input: string = '[data-test-id="Ein_Label-text-input"]';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        TextEditorComponent,
        ConvertForDataTestPipe,
        MockComponent(ValidationErrorComponent),
      ],
      imports: [MatFormFieldModule, MatInputModule, ReactiveFormsModule, BrowserAnimationsModule],
      providers: [
        {
          provide: NgControl,
          useValue: new UntypedFormControl(),
        },
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(TextEditorComponent);
    component = fixture.componentInstance;
    component.label = 'Ein Label';
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should exist for input', () => {
    const element: HTMLElement = fixture.nativeElement.querySelector(input);

    expect(element).toBeInstanceOf(HTMLElement);
  });

  it('should use fixed subscription sizing by default', () => {
    expect(component.subscriptSizing).toBe('fixed');
  });

  describe('onClear', () => {
    it('should write null value', () => {
      component.writeValue = jest.fn();

      component.onClear();

      expect(component.writeValue).toBeCalledWith(null);
    });
  });

  describe('ngOnInit', () => {
    describe('showClearButton$', () => {
      it('should emit true', (done) => {
        component.clearable = true;
        component.fieldControl.setValue('test');

        component.ngOnInit();

        component.showClearButton$.subscribe((value) => {
          expect(value).toBeTruthy();
          done();
        });
      });

      it('should emit false when not clearable', (done) => {
        component.clearable = false;
        component.fieldControl.setValue('test');

        component.ngOnInit();

        component.showClearButton$.subscribe((value) => {
          expect(value).toBeFalsy();
          done();
        });
      });

      it.each([null, '', ' '])('should emit false when content = %s', (content: string, done) => {
        component.clearable = true;
        component.fieldControl.setValue(content);

        component.ngOnInit();

        component.showClearButton$.subscribe((value) => {
          expect(value).toBeFalsy();
          done();
        });
      });
    });
  });
});
