/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { mock } from '@alfa-client/test-utils';
import { NotificationsSendFor, UserSettingsService } from '@alfa-client/user-settings-shared';
import { MockComponent } from 'ng-mocks';
import { UserSettingsEmailBenachrichtigungContainerComponent } from './user-settings-email-benachrichtigung-container.component';
import { UserSettingsEmailBenachrichtigungComponent } from './user-settings-email-benachrichtigung/user-settings-email-benachrichtigung.component';

describe('UserSettingsEmailBenachrichtigungContainerComponent', () => {
  let component: UserSettingsEmailBenachrichtigungContainerComponent;
  let fixture: ComponentFixture<UserSettingsEmailBenachrichtigungContainerComponent>;

  const userSettingsService = { ...mock(UserSettingsService), setUserSettings: jest.fn() };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        UserSettingsEmailBenachrichtigungContainerComponent,
        MockComponent(UserSettingsEmailBenachrichtigungComponent),
      ],
      providers: [
        {
          provide: UserSettingsService,
          useValue: userSettingsService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(UserSettingsEmailBenachrichtigungContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call userSettings service', () => {
      component.ngOnInit();

      expect(userSettingsService.getUserSettings).toHaveBeenCalled();
    });
  });

  describe('change notifications send for', () => {
    it('should call service with ALL', () => {
      component.changeNotificationsSendFor(true);

      expect(userSettingsService.setUserSettings).toHaveBeenCalledWith({
        notificationsSendFor: NotificationsSendFor.ALL,
      });
    });

    it('should call service with NONE', () => {
      component.changeNotificationsSendFor(false);

      expect(userSettingsService.setUserSettings).toHaveBeenCalledWith({
        notificationsSendFor: NotificationsSendFor.NONE,
      });
    });
  });
});
