/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  createEmptyStateResource,
  createStateResource,
  HasLinkPipe,
} from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  getElementFromFixture,
  Mock,
  mock,
  useFromMock,
} from '@alfa-client/test-utils';
import { UserSettingsLinkRel } from '@alfa-client/user-settings-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { SlideToggleComponent } from 'libs/ui/src/lib/ui/slide-toggle/slide-toggle.component';
import {
  NotificationsSendFor,
  UserSettings,
} from 'libs/user-settings-shared/src/lib/user-settings.model';
import {
  createUserSettings,
  createUserSettingsResource,
} from 'libs/user-settings-shared/test/user-settings';
import { MockComponent } from 'ng-mocks';
import { UserSettingsEmailBenachrichtigungComponent } from './user-settings-email-benachrichtigung.component';

describe('UserSettingsEmailBenachrichtigungComponent', () => {
  let component: UserSettingsEmailBenachrichtigungComponent;
  let fixture: ComponentFixture<UserSettingsEmailBenachrichtigungComponent>;

  const notificationSensForToggle: string = getDataTestIdOf('email-benachrichtigung');

  const valueChangedEmitter: Mock<EventEmitter<boolean>> = {
    ...mock(EventEmitter),
    emit: jest.fn(),
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HasLinkPipe,
        UserSettingsEmailBenachrichtigungComponent,
        MockComponent(SlideToggleComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(UserSettingsEmailBenachrichtigungComponent);
    component = fixture.componentInstance;
    component.userSettings = createEmptyStateResource();
    component.valueChanged = useFromMock(valueChangedEmitter);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('toggle', () => {
    it('should be active if notificationsSendFor is ALL', () => {
      const userSettings: UserSettings = {
        ...createUserSettings(),
        notificationsSendFor: NotificationsSendFor.ALL,
      };

      const result: boolean = component.isChecked(userSettings);

      expect(result).toBeTruthy();
    });

    it('should be not active if notificationsSendFor is NONE', () => {
      const userSettings: UserSettings = {
        ...createUserSettings(),
        notificationsSendFor: NotificationsSendFor.NONE,
      };

      const result: boolean = component.isChecked(userSettings);

      expect(result).toBeFalsy();
    });

    it('should emit value if toggled', () => {
      component.userSettings = createStateResource(
        createUserSettingsResource([UserSettingsLinkRel.EDIT]),
      );
      fixture.detectChanges();

      dispatchEventFromFixture(fixture, notificationSensForToggle, 'valueChanged');

      expect(component.valueChanged.emit).toHaveBeenCalled();
    });

    it('should hide if edit link is missing', () => {
      component.userSettings = createStateResource(createUserSettingsResource());
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, notificationSensForToggle);

      expect(element).not.toBeInTheDocument();
    });

    it('should show if edit link exist', () => {
      component.userSettings = createStateResource(
        createUserSettingsResource([UserSettingsLinkRel.EDIT]),
      );
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, notificationSensForToggle);

      expect(element).toBeInTheDocument();
    });
  });
});
