/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { KeyValue } from '@angular/common';
import { ChangeDetectionStrategy, Component, Input } from '@angular/core';
import { isArray, isObject, isString } from 'lodash-es';

@Component({
  selector: 'alfa-vorgang-detail-form-data-table',
  templateUrl: './vorgang-detail-form-data-table.component.html',
  styleUrls: ['./vorgang-detail-form-data-table.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class VorgangDetailFormDataTableComponent {
  @Input() formData: ReadonlyMap<number, string> | object;

  readonly valueKey = 'value';
  readonly labelKey = 'label';

  public isArray = isArray;
  public isObject = isObject;

  public originalOrder = (a: KeyValue<number, string>, b: KeyValue<number, string>): number => {
    return 0;
  };

  public isExpandable(value: unknown): boolean {
    return this.hasValue(value) && !isString((<FormField>value).value);
  }

  private hasValue(value: unknown): boolean {
    return value.hasOwnProperty(this.valueKey);
  }

  public getLabel(row: any): string {
    return this.hasLabel(row.value) ? row.value.label : row.key;
  }

  hasLabel(value: unknown): boolean {
    return value.hasOwnProperty(this.labelKey);
  }
}

export interface FormField {
  value: object | string;
  label?: string;
}
