import {
  ChangeDetectorRef,
  Component,
  Input,
  OnChanges,
  QueryList,
  SimpleChanges,
  ViewChild,
  ViewChildren,
} from '@angular/core';
import { MatMenuTrigger } from '@angular/material/menu';
import { isNotUndefined } from '@alfa-client/tech-shared';
import { MenuItemComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { has } from 'lodash-es';

@Component({
  selector: 'alfa-vorgang-detail-more-menu',
  templateUrl: './vorgang-detail-more-menu.component.html',
  styleUrls: ['./vorgang-detail-more-menu.component.scss'],
})
export class VorgangDetailMoreMenuComponent implements OnChanges {
  @Input() vorgangWithEingang: VorgangWithEingangResource;

  @ViewChild('menuTrigger') trigger: MatMenuTrigger;
  @ViewChildren(MenuItemComponent) menuItems!: QueryList<MenuItemComponent>;

  buttonEnabled: boolean = false;

  readonly vorgangWithEingangLinkRel = VorgangWithEingangLinkRel;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnChanges(changes: SimpleChanges): void {
    if (has(changes, 'vorgangWithEingang')) {
      this.vorgangWithEingangChanged();
    }
  }

  public vorgangWithEingangChanged(): void {
    this.changeDetectorRef.detectChanges();
    this.buttonEnabled = this.isButtonEnabled();
  }

  public isButtonEnabled(): boolean {
    return this.menuItems?.length > 0;
  }

  public closeMenu(): void {
    if (isNotUndefined(this.trigger)) {
      this.trigger.closeMenu();
    }
  }
}
