/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { CommandResource, isDone } from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { WiedervorlageService } from '@alfa-client/wiedervorlage-shared';
import { Observable } from 'rxjs';
import { filter, tap } from 'rxjs/operators';
import { WiedervorlageFormService } from '../wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-form/wiedervorlage.formservice';

@Component({
  selector: 'alfa-submit-wiedervorlage-button',
  templateUrl: './submit-wiedervorlage-button.component.html',
  styleUrls: ['./submit-wiedervorlage-button.component.scss'],
})
export class SubmitWiedervorlageButtonComponent {
  @Input() showAsIconButton: boolean = false;

  submitInProgress$: Observable<StateResource<any>>;

  constructor(
    private wiedervorlageService: WiedervorlageService,
    private formService: WiedervorlageFormService,
    private navigationService: NavigationService,
    private route: ActivatedRoute,
  ) {
    this.submitInProgress$ = this.wiedervorlageService.getSubmitInProgress();
  }

  submit(): void {
    this.submitInProgress$ = this.formService.submit().pipe(
      filter((command) => !command.loading),
      tap((commandWithError) => this.navigateIfCommandIsDone(commandWithError)),
    );
  }

  navigateIfCommandIsDone(command: StateResource<CommandResource>): void {
    if (!command.loading && isDone(command.resource)) {
      this.navigateToVorgangDetailPage();
    }
  }

  private navigateToVorgangDetailPage(): void {
    this.navigationService.navigateRelativeTo('../../', this.route);
  }
}
