package de.ozgcloud.alfa.historie;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.user.UserId;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class AssignedUserChangeHistoryBuilder extends ChangeHistoryBuilder<AssignedUserChangeHistoryBuilder> {

	static final String BODY_PROPERTY_ASSIGNED_USER = "assignedTo";

	private UserProfileCache userProfileCache;

	public static AssignedUserChangeHistoryBuilder builder() {
		return new AssignedUserChangeHistoryBuilder();
	}

	public AssignedUserChangeHistoryBuilder withUserProfileCache(UserProfileCache userProfileCache) {
		this.userProfileCache = userProfileCache;
		return this;
	}

	@Override
	boolean isRelevant(Command command) {
		return command.getCommandOrder().equals(CommandOrder.ASSIGN_USER);
	}

	@Override
	CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious) {
		return new CommandWithChangeValues(
				commandWithPrevious.getCommand(),
				getAssignedUserBeforeChange(commandWithPrevious),
				getAssignedUserAfterChange(commandWithPrevious));
	}

	String getAssignedUserBeforeChange(CommandWithPrevious commandWithPrevious) {
		return Optional.ofNullable(commandWithPrevious.getPrevious())
				.map(this::getAssignedUserFullNameFromCommand)
				.orElse(StringUtils.EMPTY);
	}

	String getAssignedUserAfterChange(CommandWithPrevious commandWithPrevious) {
		return getAssignedUserFullNameFromCommand(commandWithPrevious.getCommand());
	}

	String getAssignedUserFullNameFromCommand(Command command) {
		var assignedUserId = UserId.from(getValueFromCommandBody(BODY_PROPERTY_ASSIGNED_USER, command));
		return userProfileCache.getUserProfile(assignedUserId).getFullName();
	}
}
