/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.Map;

import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.alfa.attachment.AttachmentMapper;
import de.ozgcloud.alfa.common.binaryfile.FileIdMapper;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;

@Mapper(uses = { FileIdMapper.class }, unmappedTargetPolicy = ReportingPolicy.WARN, //
		unmappedSourcePolicy = ReportingPolicy.WARN, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
abstract class WiedervorlageMapper {

	@Autowired
	private GrpcObjectMapper grpcObjectMapper;

	static final String ID = "id";
	static final String DONE = "done";
	static final String CREATED_BY = "createdBy";
	static final String CREATED_BY_NAME = "createdByName";
	static final String CREATED_AT = "createdAt";
	static final String BETREFF = "betreff";
	static final String BESCHREIBUNG = "beschreibung";
	static final String FRIST = "frist";
	static final String ATTACHMENTS = "attachments";

	public Wiedervorlage fromItem(GrpcVorgangAttachedItem item) {
		return fromItemMap(grpcObjectMapper.mapFromGrpc(item.getItem()), item.getVersion());
	}

	Wiedervorlage fromItemMap(Map<String, Object> itemMap, long version) {
		return Wiedervorlage.builder()
				.id((String) itemMap.get(ID))
				.version(version)
				.done(Boolean.valueOf((String) itemMap.get(DONE)))
				.createdBy((String) itemMap.get(CREATED_BY))
				.createdAt(ZonedDateTime.parse((String) itemMap.get(CREATED_AT)))
				.betreff((String) itemMap.get(BETREFF))
				.beschreibung((String) itemMap.get(BESCHREIBUNG))
				.frist(LocalDate.parse((String) itemMap.get(FRIST)))
				.attachments(AttachmentMapper.mapAttachments(itemMap.get(ATTACHMENTS)))
				.build();
	}
}