/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;

import java.util.List;
import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import de.ozgcloud.vorgang.vorgang.GrpcEingang;
import de.ozgcloud.vorgang.vorgang.GrpcFormData;
import de.ozgcloud.vorgang.vorgang.GrpcFormField;

@SpringBootTest
class VorgangWithEingangMapperITCase {

	@Autowired
	private VorgangWithEingangMapper mapper;

	@Nested
	class TestToVorgangWithEingang {

		@Nested
		class TestMappingFormData {

			@Test
			void shouldMapListOfStrings() {
				var vorgang = mapper.toVorgangWithEingang(GrpcVorgangWithEingangTestFactory.createWithEingang(buildEingang()));

				assertThat(vorgang.getEingang().getFormData()).containsKey("TEST");
				assertThat(getMapEntry(vorgang, "TEST")).containsEntry("value", List.of("VALUE1", "VALUE2"));
			}

			private GrpcEingang buildEingang() {
				return GrpcEingangTestFactory.createBuilder().setFormData(GrpcFormData.newBuilder()
								.addField(GrpcFormField.newBuilder().setName("TEST").setValue("VALUE1").build())
								.addField(GrpcFormField.newBuilder().setName("TEST").setValue("VALUE2").build()))
						.build();
			}

			@SuppressWarnings("unchecked")
			private Map<String, Object> getMapEntry(VorgangWithEingang vorgang, String key) {
				return MapUtils.getMap(vorgang.getEingang().getFormData(), key);
			}
		}
	}
}