package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.xoev.xdomea.AnwendungsspezifischeErweiterungType;
import de.xoev.xdomea.FeldgruppeType;

class AnwendungsspezifischeErweiterungTypeCreatorTest {

	@Spy
	@InjectMocks
	private AnwendungsspezifischeErweiterungTypeCreator creator;

	@Mock
	private FeldGruppeTypeCreator feldGruppeTypeCreator;

	@Nested
	class TestCreate {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final FeldgruppeType feldgruppeType = new FeldgruppeType();

		@BeforeEach
		void init() {
			when(feldGruppeTypeCreator.create(vorgang)).thenReturn(feldgruppeType);
		}

		@Test
		void shouldHaveKennung() {
			var anwendungsspezifischeErweiterung = create();

			assertThat(anwendungsspezifischeErweiterung.getKennung()).isEqualTo(AnwendungsspezifischeErweiterungTypeCreator.KENNUNG);
		}

		@Test
		void shouldHaveName() {
			var anwendungsspezifischeErweiterung = create();

			assertThat(anwendungsspezifischeErweiterung.getName()).isEqualTo(AnwendungsspezifischeErweiterungTypeCreator.NAME);
		}

		@Test
		void shouldCallFeldgrupperMapper() {
			create();

			verify(feldGruppeTypeCreator).create(vorgang);
		}

		@Test
		void shouldHaveFeldGruppe() {
			var anwendungsspezifischeErweiterung = create();

			assertThat(anwendungsspezifischeErweiterung.getFeldgruppe()).hasSize(1).first().isEqualTo(feldgruppeType);
		}

		private AnwendungsspezifischeErweiterungType create() {
			return creator.create(vorgang);
		}
	}

}