package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.export.AllgemeineMetadatenTypeTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;

public class ExportVorgangServiceTest {

	private final static String VORGANG_ID = "DUMMY_ID";
	private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

	@Spy
	@InjectMocks
	private ExportVorgangService exportVorgangService;

	@Mock
	private VorgangService vorgangService;

	@Nested
	class TestGetVorgang {

		@BeforeEach
		void init() {
			when(vorgangService.findVorgangWithEingang(VORGANG_ID)).thenReturn(vorgang);
		}

		@Test
		void shouldCallVorgangService() {
			exportVorgangService.getVorgang(VORGANG_ID);

			verify(vorgangService).findVorgangWithEingang(VORGANG_ID);
		}

		@Test
		void shouldReturnVorgangWithEingang() {
			assertThat(exportVorgangService.getVorgang(VORGANG_ID)).isEqualTo(vorgang);
		}
	}

	@Nested
	class TestCreateAkteType {

		@Test
		void shouldCallCreateIdentifikationObjektType() {
			exportVorgangService.createAkteType(vorgang);

			verify(exportVorgangService).createIdentifikationObjektType();
		}

		@Test
		void shouldSetIdentifikationObjektType() {
			var identifikationObjektType = IdentifikationObjektTypeTestFactory.create();
			doReturn(identifikationObjektType).when(exportVorgangService).createIdentifikationObjektType();

			var akteType = exportVorgangService.createAkteType(vorgang);

			assertThat(akteType.getIdentifikation()).isEqualTo(identifikationObjektType);
		}

		@Test
		void shouldCallCreateAllgemeineMetadatenType() {
			exportVorgangService.createAkteType(vorgang);

			verify(exportVorgangService).createAllgemeineMetadatenType(vorgang);
		}

		@Test
		void shouldSetAllgemeineMetadatenType() {
			var allgemeineMetadatenType = AllgemeineMetadatenTypeTestFactory.create();
			doReturn(allgemeineMetadatenType).when(exportVorgangService).createAllgemeineMetadatenType(vorgang);

			var akteType = exportVorgangService.createAkteType(vorgang);

			assertThat(akteType.getAllgemeineMetadaten()).isEqualTo(allgemeineMetadatenType);
		}
	}

	@Nested
	class TestCreateIdentifikationObjektType {

		@Test
		void shouldSetID() {
			var identifikationObjektType = exportVorgangService.createIdentifikationObjektType();

			assertThat(identifikationObjektType.getID()).isNotBlank();
		}
	}

	@Nested
	class TestCreateAllgemeineMetadatenType {

		@Test
		void shouldSetKennzeichen() {
			var allgemeineMetadatenType = exportVorgangService.createAllgemeineMetadatenType(vorgang);

			assertThat(allgemeineMetadatenType.getKennzeichen()).isEqualTo(VorgangHeaderTestFactory.AKTENZEICHEN);
		}

		@Test
		void shouldSetEmptyKennzeichen() {
			var akteType = exportVorgangService.createAkteType(VorgangWithEingangTestFactory.createBuilder().aktenzeichen(null).build());

			assertThat(akteType.getAllgemeineMetadaten().getKennzeichen()).isEmpty();
		}
	}
}
